<?php
if (!defined('ABSPATH')) {
	exit; // Exit if accessed directly.
}
if (!class_exists('UBP_Ultimate_Box_Main_Settings')) {
	class UBP_Ultimate_Box_Main_Settings
	{
		public function __construct()
		{
			add_action('admin_menu', array($this, 'register_settings_page'));
			add_action('admin_init', array($this, 'register_settings'));
			add_action('admin_footer', array($this, 'css'));
			if ('disabled' !== get_option('ubp_variable_product_addon_enable')) {
				add_filter('ubp_box_product_addon_search_args', array($this, 'box_product_addon_search_args'), 10, 1);
			}
		}
		public function register_settings_page()
		{
			add_menu_page(esc_html__('Mix & Match Box Product', 'wc-ubp'), esc_html__('Mix & Match Box', 'wc-ubp'), 'manage_options', 'mix-match-box', array($this, 'menu_page_callback'), 'dashicons-products', 57);
			add_submenu_page('mix-match-box', esc_html__('Settings', 'wc-ubp'), esc_html__('Settings', 'wc-ubp'), 'manage_options', 'mix-match-box', array($this, 'menu_page_callback'));
			add_submenu_page('mix-match-box', esc_html__('Addon Plugins', 'wc-ubp'), esc_html__('Addon Plugins', 'wc-ubp'), 'manage_options', 'mix-match-addons', array($this, 'mix_match_addons_func'));
		}
		public function register_settings()
		{
			register_setting('ubp_mix_match_box_settings', 'ubp_mix_match_box_columns_layout');
			register_setting('ubp_mix_match_box_settings', 'ubp_mix_match_box_short_description');
			register_setting('ubp_mix_match_box_settings', 'ubp_mix_match_box_price_label');
			register_setting('ubp_mix_match_box_settings', 'ubp_mix_match_box_bg_color');
			register_setting('ubp_mix_match_box_settings', 'ubp_mix_match_box_border_color');
			register_setting('ubp_mix_match_box_settings', 'ubp_mix_match_box_max_posts_per_page');
			register_setting('ubp_mix_match_box_settings', 'ubp_is_category_filter_addon_active');
			register_setting('ubp_mix_match_box_settings', 'ubp_variable_product_addon_enable');
			register_setting('ubp_mix_match_box_settings', 'ubp_variable_product_addon_allow_variations');
			register_setting('ubp_mix_match_box_settings', 'ubp_mix_match_box_enable_shortcode');
		}
		public function menu_page_callback()
		{
?>
			<div class="wrap">
				<h1><?php _e('Mix & Match Box Product', 'wc-ubp'); ?></h1>
				<form action="options.php" method="post">
					<?php settings_errors(); ?>
					<?php settings_fields('ubp_mix_match_box_settings');
					do_settings_sections('ubp_mix_match_box_settings'); ?>
					<?php do_action('wc-ubp-plugin-settings'); ?>
					<table class="form-table">
						<tr>
							<th>
								<label for="ubp_is_category_filter_addon_active"><?php esc_html_e('Enable Category Filter Addon', 'wc-ubp'); ?></label>
							</th>
							<td>
								<?php $value = get_option('ubp_is_category_filter_addon_active'); ?>
								<select name="ubp_is_category_filter_addon_active" id="ubp_is_category_filter_addon_active">
									<option value="yes" <?php selected($value, 'yes', true); ?>><?php esc_html_e('Yes', 'wc-ubp'); ?></option>
									<option value="no" <?php selected($value, 'no', true); ?>><?php esc_html_e('No', 'wc-ubp'); ?></option>
								</select>
								<p><?php esc_html_e('Do you want to enable category filter addon on all products?', 'wc-ubp'); ?></p>
							</td>
						</tr>
						<tr>
							<th><label for="ubp_mix_match_box_columns_layout"><?php esc_html_e('Box Items Column Layout', 'wc-ubp'); ?></label></th>
							<td><?php $value = get_option('ubp_mix_match_box_columns_layout'); ?>
								<p><label><input type="radio" name="ubp_mix_match_box_columns_layout" value="grid" <?php checked($value, 'grid'); ?>>
										<?php esc_html_e('Grid Layout', 'wc-ubp'); ?></label></p>
								<p><label><input type="radio" name="ubp_mix_match_box_columns_layout" value="masonry" <?php checked($value, 'masonry'); ?>>
										<?php esc_html_e('Masonry Layout', 'wc-ubp'); ?></label></p>
							</td>
						</tr>
						<tr>
							<th><label for="ubp_mix_match_box_short_description"><?php esc_html_e('Show Product Short Description', 'wc-ubp'); ?></label></th>
							<td><?php $value = get_option('ubp_mix_match_box_short_description'); ?>
								<select name="ubp_mix_match_box_short_description" id="ubp_mix_match_box_short_description">
									<option value="above_title" <?php selected($value, 'above_title', true) ?>><?php esc_html_e('Above the box product title', 'wc-ubp'); ?></option>
									<option value="below_title" <?php selected($value, 'below_title', true) ?>><?php esc_html_e('Below the box product title', 'wc-ubp'); ?></option>
									<option value="below_layout" <?php selected($value, 'below_layout', true) ?>><?php esc_html_e('Below the box products layout', 'wc-ubp'); ?></option>
								</select>
							</td>
						</tr>

						<tr>
							<th><label for="ubp_mix_match_box_max_posts_per_page"><?php esc_html_e('Products Per Page', 'wc-ubp'); ?></label></th>
							<td><?php $value = get_option('ubp_mix_match_box_max_posts_per_page');  ?>
								<p><input type="number" name="ubp_mix_match_box_max_posts_per_page" id="ubp_mix_match_box_max_posts_per_page" value="<?php echo esc_attr($value); ?>" class="regular-text" placeholder="<?php esc_html_e('Box items per page', 'wc-ubp'); ?>" min="0">
								<p>
							</td>
						</tr>
						<tr>
							<th><label for="ubp_mix_match_box_price_label"><?php esc_html_e('Box Price Label', 'wc-ubp'); ?></label></th>
							<td><?php $value = get_option('ubp_mix_match_box_price_label');  ?>
								<p><input type="text" name="ubp_mix_match_box_price_label" id="ubp_mix_match_box_price_label" value="<?php echo esc_attr($value); ?>" class="regular-text" placeholder="<?php esc_html_e('Box Total: ', 'wc-ubp'); ?>">
								<p>
							</td>
						</tr>
						<tr>
							<th><label for="ubp_mix_match_box_bg_color"><?php esc_html_e('Boxes Background Color', 'wc-ubp'); ?></label></th>
							<td><?php $value = get_option('ubp_mix_match_box_bg_color');
								$value = empty($value) ? '#8224e3' : $value; ?>
								<p><input type="text" name="ubp_mix_match_box_bg_color" id="ubp_mix_match_box_bg_color" value="<?php echo esc_attr($value); ?>">
								<p>
							</td>
						</tr>
						<tr>
							<th><label for="ubp_mix_match_box_border_color"><?php esc_html_e('Boxes Border Color', 'wc-ubp'); ?></label></th>
							<td><?php $value = get_option('ubp_mix_match_box_border_color');
								$value = empty($value) ? '#8224e3' : $value; ?>
								<p><input type="text" name="ubp_mix_match_box_border_color" id="ubp_mix_match_box_border_color" value="<?php echo esc_attr($value); ?>">
								<p>
							</td>
						</tr>
						<tr>
							<th><label for="ubp_variable_product_addon_enable"><?php esc_html_e('Enable/Disable Variable Products in Box Products', 'wc-ubp'); ?></label></th>
							<td>
								<?php
								$value = get_option('ubp_variable_product_addon_enable');
								$value = 'disabled' === $value ? $value : 'enabled';
								?>
								<p><label><input type="radio" name="ubp_variable_product_addon_enable" value="enabled" <?php checked($value, 'enabled'); ?>>
										<?php esc_html_e('Enable', 'wc-ubp'); ?></label></p>
								<p><label><input type="radio" name="ubp_variable_product_addon_enable" value="disabled" <?php checked($value, 'disabled'); ?>>
										<?php esc_html_e('Disable', 'wc-ubp'); ?></label></p>
							</td>
						</tr>
						<tr>
							<th><label for="ubp_variable_product_addon_allow_variations"><?php esc_html_e('Allow Single Variations in Box Product', 'wc-ubp'); ?></label></th>
							<td>
								<?php
								$value = get_option('ubp_variable_product_addon_allow_variations');
								?>
								<p><label><input type="checkbox" name="ubp_variable_product_addon_allow_variations" <?php checked($value, 'on'); ?>></label></p>
							</td>
						</tr>
						<tr>
							<th><label for="ubp_mix_match_box_enable_shortcode"><?php esc_html_e('Enable/Disable short code for Box Products', 'wc-ubp'); ?></label></th>
							<td>
								<?php
								$value = get_option('ubp_mix_match_box_enable_shortcode');
								$value = 'enabled' === $value ? $value : 'disabled';
								?>
								<p><label><input type="radio" name="ubp_mix_match_box_enable_shortcode" value="disabled" <?php checked($value, 'disabled'); ?>>
										<?php esc_html_e('Disable', 'wc-ubp'); ?></label></p>

								<p>
								<p><label><input type="radio" name="ubp_mix_match_box_enable_shortcode" value="enabled" <?php checked($value, 'enabled'); ?>>
										<?php esc_html_e('Enable', 'wc-ubp'); ?></label></p>
								Note. Enabling shortcode allows to load scripts and styles of box product on all pages.
								</p>

							</td>
						</tr>
					</table>
					<?php submit_button(); ?>
				</form>
			</div>
		<?php
		}
		public function mix_match_addons_func()
		{
		?>
			<div class="wrap mix_match_addons">
				<h1><?php esc_html_e('Mix & Match Box Product Addon Plugins', 'wc-ubp'); ?></h1><br>
				<div class="upb-row">
					<div class="ubp-col-2">
						<div class="ubp-inner">
							<span>
								<img src="<?php echo WOO_UBP_URL; ?>assets/images/box_subscription.jpg" />
							</span>
							<span class="ubp_caption">
								<span class="ubp_title">
									<?php esc_html_e('Mix & Match Compability Addon For Woocommerce Subscriptions', 'wc-ubp') ?>
								</span>
								<span>
									<a href="https://codecanyon.net/item/mix-match-pro-addon-for-subscription-plugin/23102423" class="button button-secondary" target="_blank"><?php esc_html_e('View Product', 'wc-ubp'); ?></a>
								</span>
							</span>
						</div>
					</div>
				</div>
			</div>
		<?php
		}
		public function css()
		{
		?>
			<style type="text/css">
				li#toplevel_page_mix-match-box div.wp-menu-image:before {
					color: #d60000 !important;
				}
			</style>
<?php
		}
		public function box_product_addon_search_args($args)
		{
			$args['tax_query'] = array(
				array(
					'taxonomy' => 'product_type',
					'field'    => 'slug',
					'terms'    => array('box_product', 'subscription', 'variable-subscription'),
					'operator' => 'NOT IN',
				)
			);
			return $args;
		}
	}
	new UBP_Ultimate_Box_Main_Settings();
}
